classdef dipoleAndMeasure < pr.dipole
    % holds a measure (e.g. ERP, ERSP...) in a linearized way.
    % the first dimension is the linearized version of the measure.
    % the second dimension of value is items (usually dipoles).
    properties
        linearizedMeasure = []; % N x M matrix, N is the given measure (ERP, ERSP...) in a linearized manner, M is the number of items (usually dipoles).               
    end % properties    
    properties (Abstract = true, Constant = true)
        numberOfMeasureDimensions % this is 1 for ERP, Spec and other 1 dimensional measures, and 2 for ERP, ITC.
        % This value is not set (determined) in this class, so it is Abstract, and it does not changce for a given measure, so it is Constant.
    end;    
    methods
        function obj = dipoleAndMeasure() % constructor
            obj = obj@pr.dipole();
        end;        
        function correlationSimilaity = getPairwiseCorrelationSimilarity(obj)
            correlationSimilaity = 1-squareform(pdist(obj.linearizedMeasure', 'correlation'));            
        end;        
        function euclideanSimilaity = getPairwiseEuclideanSimilarity(obj) % could be negative as it is -euclidean distance
            euclideanSimilaity = -squareform(pdist(obj.linearizedMeasure'));
        end;        
        function estimatedMutualInformation = getPairwiseMutualInformationSimilarity(obj)
            correlationSimilaity = 1-squareform(pdist(obj.linearizedMeasure', 'correlation'));
            
            % when two dipoles are associated with one IC, their correlation becomes exactly 1 and
            % create numerical issues. That is why here we place it to a value slightly less than 1.
            correlationSimilaity(correlationSimilaity == 1) = 0.98;
            
            estimatedMutualInformation = (1/2) * log2( 1 ./ (1-correlationSimilaity .^2) ) .* sign(correlationSimilaity);
            
            % remove NaNs on the diagonal, we cannot use minus (-) since Nan - Nan = Nan
            for i=1:size(estimatedMutualInformation, 1)
                estimatedMutualInformation(i,i) = 0;
            end;
            
        end;        
        function [linearizedProjectedMeasure dipoleDensity] = projectTo(obj, position, standardDeviationOfEstimatedDipoleLocation)
            % position is N x 3 array of location to which the projection should occur
            % returns an L x N array for N projected positions.
            if nargin<3
                standardDeviationOfEstimatedDipoleLocation = 12;
            end;
            
            linearizedProjectedMeasure = zeros(size(obj.linearizedMeasure,1), size(position, 1));
            dipoleDensity = zeros(1, size(position, 1));
            
            for i=1:size(position, 1)
                [linearizedProjectedMeasure(:,i) dipoleDensity(i)] = project_eeg_measure_to_space(position(i,:), obj.location, obj.linearizedMeasure, standardDeviationOfEstimatedDipoleLocation);
            end;
        end;
    end;
end